@@ -1,11 +1,8 @@
 from . import core
 from .util.wsgi import ClosingIterator
-import fresco.flash
 
 
-__all__ = 'FlashMiddleware', 'context_middleware', 'XForwarded'
-
-FlashMiddleware = fresco.flash.FlashMiddleware
+__all__ = 'context_middleware', 'XForwarded'
 
 
 def context_middleware(app, frescoapp):
@@ -39,14 +36,14 @@ class XForwarded(object):
 
     Example::
 
-        >>> from fresco import FrescoApp, GET, Response
+        >>> from fresco import FrescoApp, context, GET, Response
         >>> from flea import TestAgent
         >>> app = FrescoApp()
         >>> @app.route('/', GET)
-        ... def view(request):
-        ...     return Response(["URL is ", request.url,
+        ... def view():
+        ...     return Response(["URL is ", context.request.url,
         ...                      "; REMOTE_ADDR is ",
-        ...                      request.remote_addr])
+        ...                      context.request.remote_addr])
         ...
         >>> r = TestAgent(XForwarded(app))
         >>> response = r.get('/',
