@@ -8,9 +8,9 @@ HTTP headers and request data (eg query string or submitted form data).
 
 import posixpath
 import re
-import sys
-from urllib import quote
-from urlparse import urlparse, urlunparse
+
+from fresco.compat import string_types, quote, urlparse, urlunparse, ParseResult
+from fresco.util.wsgi import environ_to_unicode
 
 import fresco
 
@@ -37,7 +37,6 @@ class Request(object):
     #: Maximum size for multipart/form-data encoded post data
     MAX_MULTIPART_SIZE = 2 * MB
 
-    _session = None
     _form = None
     _files = None
     _query = None
@@ -135,15 +134,14 @@ class Request(object):
             >>> from fresco import FrescoApp
             >>> with FrescoApp().requestcontext('/?animal=moose') as c:
             ...     c.request.query['animal']
-            u'moose'
+            'moose'
 
         This property always returns querystring data, regardless of the
         request method used.
         """
         if self._query is None:
-            self._query = MultiDict(
-                parse_querystring(self.environ.get('QUERY_STRING'))
-            )
+            query = environ_to_unicode(self.environ.get('QUERY_STRING', ''))
+            self._query = MultiDict(parse_querystring(query, self.charset))
 
         return self._query
 
@@ -232,8 +230,8 @@ class Request(object):
         """
         Return the path component of the requested URL
         """
-        return self.environ.get('SCRIPT_NAME', '') \
-                + self.environ.get('PATH_INFO', '')
+        return environ_to_unicode(self.environ.get('SCRIPT_NAME', '') +
+                                  self.environ.get('PATH_INFO', ''))
 
     @property
     def url(self):
@@ -258,9 +256,10 @@ class Request(object):
             'http://example.com/animals'
         """
         scheme, netloc, path, params, query, frag = self.parsed_url
+
         return urlunparse((scheme, netloc,
-                           self.environ.get('SCRIPT_NAME', ''),
-                           '', '', ''))
+                           quote(self.script_name.encode(self.charset)), '',
+                           '', ''))
 
     @property
     def parsed_url(self):
@@ -278,15 +277,17 @@ class Request(object):
             ...          'https://example.com/animals/view?name=lion') as c:
             ...     c.request.parsed_url
             ('https', 'example.com', '/animals/view', '', 'name=lion', '')
+
+        Components are returned as a unicode strings
         """
         env = self.environ.get
-        script_name = env("SCRIPT_NAME", "")
-        path_info = env("PATH_INFO", "")
-        query_string = env("QUERY_STRING", "")
-        scheme = env('wsgi.url_scheme', 'http')
+        script_name = quote(self.script_name.encode(self.charset))
+        path_info = quote(self.path_info.encode(self.charset))
+        query_string = self.query_string
+        scheme = environ_to_unicode(env('wsgi.url_scheme', 'http'))
 
         try:
-            host = self.environ['HTTP_HOST']
+            host = environ_to_unicode(self.environ['HTTP_HOST'])
             if ':' in host:
                 host, port = host.split(':', 1)
             else:
@@ -295,8 +296,8 @@ class Request(object):
                 else:
                     port = '80'
         except KeyError:
-            host = self.environ['SERVER_NAME']
-            port = self.environ['SERVER_PORT']
+            host = environ_to_unicode(self.environ['SERVER_NAME'])
+            port = environ_to_unicode(self.environ['SERVER_PORT'])
 
         if (scheme == 'http' and port == '80') \
             or (scheme == 'https' and port == '443'):
@@ -304,7 +305,7 @@ class Request(object):
         else:
             netloc = host + ':' + port
 
-        return (
+        return ParseResult(
             scheme,
             netloc,
             script_name + path_info,
@@ -316,45 +317,59 @@ class Request(object):
     @property
     def path_info(self):
         """
-        The decoded path info
+        The PATH_INFO value as a unicode string
+
+        Note that PATH_INFO is already unquoted by the server
         """
-        if sys.version_info[0] >= 3:
-            return self.environ.get('PATH_INFO', '')\
-                       .encode('latin-1').decode(self.charset)
-        else:
-            return self.environ.get('PATH_INFO', '').decode(self.charset)
+        return environ_to_unicode(self.environ.get('PATH_INFO', ''),
+                                  self.charset)
 
     @property
     def script_name(self):
+        """\
+        The SCRIPT_NAME value as a unicode string
+
+        Note that SCRIPT_NAME is already unquoted by the server
         """
-        The decoded path info
+        return environ_to_unicode(self.environ.get('SCRIPT_NAME', ''),
+                                  self.charset)
+
+    @property
+    def query_string(self):
+        """\
+        The QUERY_STRING value as a unicode string
         """
-        if sys.version_info[0] >= 3:
-            return self.environ.get('SCRIPT_NAME', '')\
-                               .encode('latin-1').decode(self.charset)
-        else:
-            return self.environ.get('SCRIPT_NAME', '').decode(self.charset)
+        try:
+            return environ_to_unicode(self.environ.get("QUERY_STRING"))
+        except AttributeError:
+            return None
 
     @property
     def referrer(self):
         """
         Return the HTTP referer header, or ``None`` if this is not available.
         """
-        return self.environ.get("HTTP_REFERER")
+        try:
+            return environ_to_unicode(self.environ.get("HTTP_REFERER"))
+        except AttributeError:
+            return None
 
     @property
     def method(self):
         """
         Return the HTTP method used for the request, eg ``GET`` or ``POST``.
         """
-        return self.environ["REQUEST_METHOD"]
+        return environ_to_unicode(self.environ["REQUEST_METHOD"])
 
     @property
     def remote_addr(self):
         """
         Return the remote address of the client
         """
-        return self.environ.get("REMOTE_ADDR", None)
+        try:
+            return environ_to_unicode(self.environ.get("REMOTE_ADDR"))
+        except AttributeError:
+            return None
 
     @property
     def session(self):
@@ -435,7 +450,7 @@ class Request(object):
         The ``query`` argument can take a dictionary, a list of ``(name,
         value)`` pairs or any other type convertable to a MultiDict::
 
-            >>> request.make_url(query=u'a=tokyo&b=milan')
+            >>> request.make_url(query='a=tokyo&b=milan')
             'http://example.com/fruitsalad/banana?a=tokyo&b=milan'
 
             >>> request.make_url(query={'a': 'tokyo', 'b': 'milan'})
@@ -465,8 +480,8 @@ class Request(object):
         parsed_url = self.parsed_url
 
         if path is not None:
-            if isinstance(path, unicode):
-                path = path.encode(fresco.DEFAULT_CHARSET)
+            if isinstance(path, bytes):
+                path = path.decode('iso-8859-1')
             if path[0] != '/':
                 path = posixpath.join(posixpath.dirname(parsed_url[2]), path)
                 path = posixpath.normpath(path)
@@ -480,16 +495,13 @@ class Request(object):
         else:
             path = parsed_url[2]
 
-        if isinstance(path, unicode):
-            path = path.encode(self.charset)
-
         path = quote(path)
 
         if query is not None:
-            if not isinstance(query, basestring):
+            if not isinstance(query, string_types):
                 query = make_query(query, **kwargs)
-            elif isinstance(query, unicode):
-                query = query.encode(self.charset)
+            elif isinstance(query, bytes):
+                query = query.decode('iso-8859-1')
                 if kwargs:
                     if query:
                         query += ';'
@@ -531,7 +543,10 @@ class Request(object):
 
         The returned URL path will be normalized:
 
-            >>> request.resolve_url('../bar')
+            >>> app = FrescoApp()
+            >>> with app.requestcontext('http://example.net/foo/bar') as c:
+            ...     c.request.resolve_url('../bar')
+            ...
             'http://example.net/bar'
 
         URLs can be resolved relative to the application's base URL (including
@@ -542,10 +557,8 @@ class Request(object):
             >>> with app.requestcontext(SCRIPT_NAME='/foo',
             ...                         PATH_INFO='bar') as c:
             ...     request = c.request
-
             >>> request.resolve_url('/baz', relative='app')
             'http://localhost/foo/baz'
-
             >>> request.resolve_url('/baz', relative='server')
             'http://localhost/baz'
 
