@@ -1,18 +1,46 @@
-import inspect
 import re
+import sys
+from collections import namedtuple
 from functools import partial
 from itertools import cycle
 from operator import attrgetter, itemgetter, methodcaller
 
-from fresco.exceptions import BadRequest
+from fresco.compat import string_types
+from fresco.exceptions import BadRequest, ResponseException
+from fresco.response import Response
+from fresco.requestcontext import context
 from fresco.util.urls import join_path
 
-__all__ = 'Pattern', 'Route', 'FormArg', 'QueryArg', 'CookieArg'
+HTTP_METHODS = GET, HEAD, POST, PUT, DELETE = \
+               'GET', 'HEAD', 'POST', 'PUT', 'DELETE'
+
+ALL_METHODS = HTTP_METHODS
+
+
+__all__ = HTTP_METHODS + ('ALL_METHODS', 'Pattern', 'Route', 'DelegateRoute',
+                          'RouteCollection', 'FormArg', 'QueryArg',
+                          'CookieArg', 'routefor')
+
+
+#: Encapsulate a pattern match on a path
+PathMatch = namedtuple('PathMatch', ['path_matched', 'path_remaining',
+                                     'args', 'kwargs'])
+
+#: Encapsulate a route traversal
+RouteTraversal = namedtuple('RouteTraversal',
+                            ['route', 'args', 'kwargs',
+                             'collections_traversed'])
 
 
 class URLGenerationError(Exception):
     """\
-    Was not possible to generate the requested URL
+    Was not possible to generate the requested URL.
+    """
+
+
+class RouteNotFound(Exception):
+    """\
+    The named route does not exist in the RouteCollection.
     """
 
 
@@ -38,8 +66,11 @@ class Pattern(object):
 
     def pathfor(self, *args, **kwargs):
         """
-        The inverse of ``match``: where possible, should generate a URL path
-        for the given positional and keyword arguments.
+        The inverse of ``match``, this should return a URL path
+        for the given positional and keyword arguments, along with any unused
+        arguments.
+
+        :return: a tuple of ``(path, remaining_args, remaining_kwargs)``
         """
         raise NotImplementedError()
 
@@ -76,7 +107,7 @@ class Converter(object):
         """
         Convert arbitrary argument ``ob`` to a string representation
         """
-        return unicode(ob)
+        return str(ob)
 
     def from_string(self, s):
         """
@@ -100,9 +131,9 @@ class IntConverter(Converter):
         return int(s)
 
 
-class UnicodeConverter(Converter):
+class StrConverter(Converter):
     """\
-    Match any string, not including a forward slash, and return a ``unicode``
+    Match any string, not including a forward slash, and return a ``str``
     value
     """
 
@@ -110,18 +141,18 @@ class UnicodeConverter(Converter):
 
     def to_string(self, s):
         """
-        Return ``s`` converted to an ``unicode`` object.
+        Return ``s`` converted to an ``str`` object.
         """
         return s
 
     def from_string(self, s):
         """
-        Return ``s`` converted to an ``unicode`` object.
+        Return ``s`` converted to an ``str`` object.
         """
-        return unicode(s)
+        return str(s)
 
 
-class AnyConverter(UnicodeConverter):
+class AnyConverter(StrConverter):
     """
     Match any one of the given string options.
     """
@@ -135,10 +166,10 @@ class AnyConverter(UnicodeConverter):
         self.pattern = '|'.join(re.escape(arg) for arg in args)
 
 
-class PathConverter(UnicodeConverter):
+class PathConverter(StrConverter):
     """\
     Match any string, possibly including forward slashes, and return a
-    ``unicode`` object.
+    ``str`` object.
     """
     pattern = r'.+'
 
@@ -154,19 +185,19 @@ class MatchAllURLsPattern(Pattern):
 
     def match(self, path):
         if path.startswith(self.path):
-            return (), {}
+            return PathMatch(self.path, path[len(self.path):], (), {})
         return None
 
     def pathfor(self, *args, **kwargs):
         assert not args and not kwargs, \
-                "MatchAllURLsPattern does not handle URL based arguments"
+                "MatchAllURLsPattern does not support URL arguments"
 
-        return self.path
+        return self.path, (), {}
 
     def add_prefix(self, prefix):
         return self.__class__(join_path(prefix, self.path))
 
-    def __unicode__(self):
+    def __str__(self):
         return '%s*' % (self.path,)
 
 
@@ -177,9 +208,9 @@ class ExtensiblePattern(Pattern):
     Synopsis::
 
         >>> from pprint import pprint
-        >>> p = ExtensiblePattern(r"/<:unicode>/<year:int>/<title:unicode>")
+        >>> p = ExtensiblePattern(r"/<:str>/<year:int>/<title:str>")
         >>> pprint(p.match('/archive/1999/blah'))
-        ((u'archive',), {'title': u'blah', 'year': 1999})
+        (('archive',), {'title': 'blah', 'year': 1999})
 
     Patterns are split on slashes into components. A component can either be a
     literal part of the path, or a pattern component in the form::
@@ -198,18 +229,19 @@ class ExtensiblePattern(Pattern):
 
     - ``int`` - converts to an integer
     - ``path`` - any path (ie can include forward slashes)
-    - ``unicode`` - any unicode string (not including forward slashes)
+    - ``str`` - any string (not including forward slashes)
+    - ``unicode`` - alias for ``str``
     - ``any`` - a string matching a list of alternatives
 
     Some examples::
 
         >>> p = ExtensiblePattern(r"/images/<:path>")
         >>> p.match('/images/thumbnails/02.jpg')
-        ((u'thumbnails/02.jpg',), {})
+        (('thumbnails/02.jpg',), {})
 
         >>> p = ExtensiblePattern("/<page:any('about', 'help')>.html")
         >>> p.match('/about.html')
-        ((), {'page': u'about'})
+        ((), {'page': 'about'})
 
         >>> p = ExtensiblePattern("/entries/<id:int>")
         >>> p.match('/entries/23')
@@ -220,7 +252,8 @@ class ExtensiblePattern(Pattern):
 
     preset_patterns = (
         ('int', IntConverter),
-        ('unicode', UnicodeConverter),
+        ('str', StrConverter),
+        ('unicode', StrConverter),
         ('path', PathConverter),
         ('any', AnyConverter),
     )
@@ -251,23 +284,31 @@ class ExtensiblePattern(Pattern):
             self.name = name
             self.converter = converter
 
-    def __init__(self, pattern, name=''):
+    def __init__(self, pattern, match_entire_path=True):
         """
         Initialize a new ``ExtensiblePattern`` object with pattern ``pattern``
-        and an optional name.
+
+        :param pattern: The pattern string, eg ``'/<id:int>/show'``
+        :param match_entire_path: Boolean. If ``True``, the entire path will be
+                                  required to match, otherwise a prefix match
+                                  will suffice.
         """
         super(ExtensiblePattern, self).__init__()
 
-        self.name = name
         self.preset_patterns = dict(self.preset_patterns)
-        self.pattern = unicode(pattern)
+        self.pattern = pattern
+        self.match_entire_path = match_entire_path
 
         self.segments = list(self._make_segments(pattern))
         self.args = [item
                      for item in self.segments
                      if item.converter is not None]
-        self.regex = re.compile(''.join(segment.regex
-                                        for segment in self.segments) + '$')
+
+        regex = ''.join(segment.regex for segment in self.segments)
+        if self.match_entire_path:
+            regex += '$'
+
+        self.regex = re.compile(regex)
 
     def _make_segments(self, s):
         r"""
@@ -279,15 +320,12 @@ class ExtensiblePattern(Pattern):
         """
 
         for item in split_iter(self.pattern_parser, self.pattern):
-            if isinstance(item, unicode):
+            if isinstance(item, string_types):
                 yield self.Segment(item, re.escape(item), None, None)
                 continue
             groups = item.groupdict()
             name, converter, args = (groups['name'], groups['converter'],
                                      groups['args'])
-            if isinstance(name, unicode):
-                # Name must be a Python identifiers
-                name = name.encode("ASCII")
             converter = self.preset_patterns[converter]
             if args:
                 args, kwargs = self.parseargs(args)
@@ -313,12 +351,12 @@ class ExtensiblePattern(Pattern):
         """
         return eval('(lambda *args, **kwargs: (args, kwargs))(%s)' % argstr)
 
-    def match(self, uri):
+    def match(self, path):
         """
-        Test ``uri`` and return a tuple of parsed ``(args, kwargs)``, or
+        Test ``path`` and return a tuple of parsed ``(args, kwargs)``, or
         ``None`` if there was no match.
         """
-        mo = self.regex.match(uri)
+        mo = self.regex.match(path)
         if not mo:
             return None
         groups = mo.groups()
@@ -338,29 +376,31 @@ class ExtensiblePattern(Pattern):
 
         args = tuple(value for name, value in groups if not name)
         kwargs = dict((name, value) for name, value in groups if name)
-        return args, kwargs
+        return PathMatch(mo.group(0), path[len(mo.group(0)):], args, kwargs)
 
     def pathfor(self, *args, **kwargs):
         """
         Example usage::
 
-            >>> p = ExtensiblePattern("/view/<name:unicode>/<revision:int>")
+            >>> p = ExtensiblePattern("/view/<name:str>/<revision:int>")
             >>> p.pathfor(name='important_document.pdf', revision=299)
-            u'/view/important_document.pdf/299'
+            '/view/important_document.pdf/299'
 
-            >>> p = ExtensiblePattern("/view/<:unicode>/<:int>")
+            >>> p = ExtensiblePattern("/view/<:str>/<:int>")
             >>> p.pathfor('important_document.pdf', 299)
-            u'/view/important_document.pdf/299'
+            '/view/important_document.pdf/299'
         """
 
         args = list(args)
+        kwargs = kwargs
         result = []
         for seg in self.segments:
             if not seg.converter:
                 result.append(seg.source)
             elif seg.name:
                 try:
-                    result.append(seg.converter.to_string(kwargs[seg.name]))
+                    result.append(seg.converter.to_string(
+                                                    kwargs.pop(seg.name)))
                 except IndexError:
                     raise URLGenerationError(
                         "Argument %r not specified for url %r" % (
@@ -376,14 +416,10 @@ class ExtensiblePattern(Pattern):
                             self.pattern,
                         )
                     )
-        return ''.join(result)
+        return ''.join(result), args, kwargs
 
     def add_prefix(self, prefix):
-
-        return self.__class__(
-            join_path(prefix, self.pattern),
-            self.name
-        )
+        return self.__class__(join_path(prefix, self.pattern))
 
     @classmethod
     def register_converter(cls, name, converter):
@@ -408,8 +444,8 @@ class ExtensiblePattern(Pattern):
     def __repr__(self):
         return '<%s %r>' % (self.__class__, self.pattern)
 
-    def __unicode__(self):
-        return u'%s' % (self.pattern,)
+    def __str__(self):
+        return '%s' % (self.pattern,)
 
 
 class Route(object):
@@ -417,14 +453,11 @@ class Route(object):
     Represent a URL routing pattern
     """
 
-    __slots__ = 'tag', 'pattern', 'methods', 'view', 'view_kwargs',\
-                'predicate', 'decorators', 'original_view'
-
     #: The default class to use for URL pattern matching
     pattern_class = ExtensiblePattern
 
     def __init__(self, pattern, methods, view, view_kwargs=None,
-                 tag='__default__', predicate=None, decorators=None, **kwargs):
+                 name=None, predicate=None, decorators=None, **kwargs):
 
         """
         :param pattern:     A string that can be compiled into a path pattern
@@ -433,7 +466,7 @@ class Route(object):
         :param view:        The view function.
         :param view_kwargs: A dictionary of default keyword arguments to pass
                             to the view callable
-        :param tag:         A tag that can later be used to retrieve the route
+        :param name:        A name that can later be used to retrieve the route
                             for URL generation
         :param predicate:   A callable that is used to decide whether to match
                             this pattern. The callable must take a ``Request``
@@ -442,44 +475,97 @@ class Route(object):
                             before invoking it
         :param kwargs:      Other keyword aruments passed through to the view
                             callable.
+
+
+        Naming routes
+        -------------
+
+        Naming routes allows you to reference routes throughout your
+        application, eg when generating URLs with
+        :function:`~fresco.core.urlfor`.
+
+        If you don't specify a ``name`` argument, the route will available by
+        its fully qualified function name (eg
+        ``'package.module.view_function'``), or by passing the function object
+        itself to ``urlfor``
+
+        Views may often be assigned to multiple routes, for example::
+
+            Route('/image', display_image, size=(1024, 768))
+            Route('/thumbnail', display_image, size=(75, 75))
+
+        If you generate a URL for the view function ``display_image``, the
+        the first declared route will always win, in this case::
+
+            >>> urlfor(display_image)
+            'http://localhost/image'
+
+        To generate URLs for the thumbnail route in this example, you must
+        explicity assign names::
+
+            Route('/image', display_image, name='image', ...)
+            Route('/thumbnail', display_image, name='image-thumbnail', ...)
+
+            >> urlfor('image')
+            'http://localhost/image'
+            >> urlfor('image-thumbnail')
+            'http://localhost/thumbnail'
+
         """
 
+        if isinstance(methods, string_types):
+            methods = [methods]
+        else:
+            # Catch the common error of not providing a valid method
+            try:
+                methods = set(methods)
+            except TypeError:
+                raise TypeError("Methods must be a string or iterable")
+
         if not isinstance(pattern, Pattern):
             pattern = self.pattern_class(pattern)
 
-        if isinstance(methods, basestring):
-            methods = [methods]
+        if name and ':' in name:
+            # Colons are reserved to act as separators
+            raise ValueError("Route names cannot contain ':'")
 
         self.pattern = pattern
         self.methods = set(methods)
         self.view_kwargs = dict(view_kwargs or {}, **kwargs)
-        for name, arg in self.view_kwargs.items():
+        for argname, arg in self.view_kwargs.items():
             if isinstance(arg, RouteKwarg):
-                arg.configure(self, name)
-        self.tag = tag
+                arg.configure(self, argname)
+
+        self.name = name
         self.predicate = predicate
         self.decorators = decorators or []
         self.set_view(view)
 
-    def __unicode__(self):
-        return u'%s %s => %r\n' % (
+    def __str__(self):
+        return '%s %s => %r\n' % (
             ' '.join(self.methods),
-            unicode(self.pattern),
+            str(self.pattern),
             self.original_view
         )
 
     def __repr__(self):
-        return u'<%s %s %r %s>' % (
+        return '<%s %s %r %s>' % (
             self.__class__.__name__,
             ','.join(self.methods),
-            unicode(self.pattern),
+            str(self.pattern),
             self.original_view
         )
 
+    def match(self, path, method):
+        if method and method not in self.methods:
+            return None
+        return self.pattern.match(path)
+
     def set_view(self, view):
         decorated_view = view
-        for d in (self.decorators or []):
-            decorated_view = d(decorated_view)
+        if not isinstance(view, string_types):
+            for d in (self.decorators or []):
+                decorated_view = d(decorated_view)
 
         self.original_view = view
         self.view = decorated_view
@@ -488,53 +574,42 @@ class Route(object):
         """
         Resolve a view spec string to the actual callable.
         """
-        if callable(self.original_view):
-            return
-        self.set_view(getattr(instance, self.original_view))
+        if isinstance(self.original_view, string_types):
+            self.set_view(getattr(instance, self.original_view))
 
     def add_prefix(self, path):
         """
         Return a copy of the Route object with the given path prepended to the
         routing pattern.
         """
-        return self.__class__(self.pattern.add_prefix(path), self.methods,
-                              view=self.original_view,
-                              view_kwargs=self.view_kwargs, tag=self.tag,
-                              predicate=self.predicate,
-                              decorators=self.decorators)
+        newroute = object.__new__(self.__class__)
+        newroute.__dict__ = dict(self.__dict__,
+                                 pattern=self.pattern.add_prefix(path))
+        return newroute
 
-    def url(self, *args, **kwargs):
+    def path(self, *args, **kwargs):
         """\
-        Return a URL corresponding to the route
+        Return a tuple of URL path component corresponding to the route and
+        remaining positional and keyword arguments
 
-        :param _request: the current request object.
-        :param _scheme: the URL scheme to use (eg 'https' or 'http').
-        :param _netloc: the network location to use (eg 'localhost:8000').
-        :param _script_name: the SCRIPT_NAME path component
-        :param _query: any query parameters, as a dict or list of
-                       ``(key, value)`` tuples.
-        :param _fragment: a URL fragment to append.
-
-        Any other arguments or keyword args are fed to the ``pathfor`` method
-        of the pattern>
+        Positional and keywords arguments are forwarded to the ``pathfor``
+        method of the pattern.
         """
-        from fresco import context
+        return self.pattern.pathfor(*args, **kwargs)
 
-        request = kwargs.pop('_request', None)
-        scheme = kwargs.pop('_request', None)
-        netloc = kwargs.pop('_netloc', None)
-        query = kwargs.pop('_query', {})
-        script_name = kwargs.pop('_script_name', None)
-        fragment = kwargs.pop('_fragment', None)
+    def route_keys(self):
+        """\
+        Generate keys by which the route should be indexed
+        """
+        if self.name:
+            yield self.name
+        yield self.original_view
 
-        if request is None:
-            request = context.request
-
-        return request.make_url(scheme=scheme, netloc=netloc,
-                                PATH_INFO=self.pattern.pathfor(*args,
-                                                               **kwargs),
-                                SCRIPT_NAME=script_name, parameters='',
-                                query=query, fragment=fragment)
+        # Also return the underlying function object for python2 unbound
+        # methods
+        func_ob = getattr(self.original_view, '__func__', None)
+        if func_ob is not None:
+            yield func_ob
 
 
 def split_iter(pattern, string):
@@ -552,30 +627,293 @@ def split_iter(pattern, string):
     yield string[pos:]
 
 
-def resolve_viewspec(viewspec, stack_depth):
+def routefor(viewspec, _app=None):
+    """\
+    Convenience wrapper around :meth:`~fresco.routing.RouteCollection.urlfor`.
     """
-    Resolve the given view spec
+    return context.app.routefor(viewspec)
 
-    :param viewspec: View specification (eg ``'mypackage.views:my_view'``)
-    :param stack_depth: How far back to look in the stack for the caller's
-                        context, for resolving imports relative to the caller.
+
+class RouteCollection(object):
+    """\
+    A collection of :class:`~fresco.routing.Route` objects, RouteCollections:
+
+    - Contain methods to configure routes, including the ability to
+      delegate URLs to other RouteCollections
+    - Can map from a request to a view
     """
-    if not isinstance(viewspec, basestring):
-        return viewspec
 
-    try:
-        module_spec, attr_spec = viewspec.split(':')
-    except ValueError:
-        raise ValueError("Expected string like 'package.module:view_func';"\
-                         " got %r'" % viewspec)
+    def __init__(self, routes=None):
+        self.__routes__ = []
+        self.__routed_views__ = {}
+        if routes:
+            for item in routes:
+                self.add_route(item)
 
-    ob = __import__(module_spec)
-    for mod in module_spec.split('.')[1:]:
-        ob = getattr(ob, mod)
+    def add_route(self, route):
+        self.__routes__.append(route)
 
-    for attr in attr_spec.split('.'):
-        ob = getattr(ob, attr)
-    return ob
+        for item in route.route_keys():
+            if item not in self.__routed_views__:
+                self.__routed_views__[item] = route
+
+        self._add_url_method(route.original_view, route)
+
+    def _add_url_method(self, func, route):
+        """\
+        Add a method available at ``func.url`` that returns a URL generated
+        from the given route
+        """
+        from fresco.core import urlfor
+        if hasattr(func, 'url'):
+            return
+        try:
+            func.url = partial(urlfor, func)
+        except AttributeError:
+            # Cannot set function attributes on bound or unbound methods.
+            # See http://www.python.org/dev/peps/pep-0232/
+            pass
+
+    def pathfor(self, viewspec, *args, **kwargs):
+        """\
+        Return the path component of the url for the given view name/function
+        spec.
+
+        :param viewspec: a view name, a reference in the form
+                         ``'package.module.viewfunction'``, or the view
+                         callable itself.
+        """
+        if isinstance(viewspec, string_types) and ':' in viewspec:
+            viewspec, remainder = viewspec.split(':', 1)
+            delegated_route = self.routefor(viewspec)
+            p1, args, kwargs = delegated_route.path(*args, **kwargs)
+            rc = delegated_route.routecollectionfactory(*args, **kwargs)
+            rc.pathfor(remainder, *args, **kwargs)
+            p2 = delegated_route.routecollectionfactory(*args, **kwargs)\
+                                .pathfor(remainder, *args, **kwargs)
+            return p1 + p2
+
+        return self.routefor(viewspec).path(*args, **kwargs)[0]
+
+    def routefor(self, viewspec):
+        """
+        Return the :class:`~fresco.routing.Route` instance associated with
+        ``viewspec``.
+
+        Views may have multiple routes bound, in this case the first bound
+        route will always take precedence.
+
+        This method does not resolve delegated routes.
+
+        :param viewspec: a view callable or a string in the form
+                         'package.module.viewfunction'
+        """
+
+        try:
+            return self.__routed_views__[viewspec]
+        except KeyError:
+            if not isinstance(viewspec, string_types):
+                raise RouteNotFound(viewspec)
+
+            modname, symbols = viewspec, []
+            while True:
+                try:
+                    modname, sym = modname.rsplit('.', 1)
+                except ValueError:
+                    raise RouteNotFound(viewspec)
+
+                symbols.append(sym)
+                module = sys.modules.get(modname, None)
+                if module:
+                    ob = module
+                    for s in reversed(symbols):
+                        ob = getattr(ob, s)
+                    try:
+                        route = self.__routed_views__[ob]
+                    except KeyError:
+                        # An unbound class method? (python 2 only)
+                        try:
+                            route = self.__routed_views__[ob.__func__]
+                        except (AttributeError, KeyError):
+                            raise RouteNotFound(viewspec)
+
+                    self.__routed_views__[viewspec] = route
+                    return route
+
+    def get_routes(self, path, method):
+        """
+        Generate routes matching the given path and method::
+
+            for rt in routecollection.get_routes('/foo/bar', GET):
+                print("Route is", rt.route)
+                print("Arguments extracted from the path:", rt.args, rt.kwargs)
+                print("RouteCollections are:", rt.collections_traversed)
+
+        :param request: a :class:`~fresco.request.Request` object
+        :param path: the URL path to match (usually this is ``PATH_INFO``)
+        :param method: the HTTP method to match (usually this is
+                       ``REQUEST_METHOD``). May be ``None``, in which case
+                       matching will be performed on the ``path`` argument
+                       only.
+
+        :return: A generator yielding ``RouteTraversal`` objects
+        """
+        for route in self.__routes__:
+
+            result = route.match(path, method)
+            if result is None:
+                continue
+
+            view_args = result.args
+            view_kwargs = {}
+            if route.view_kwargs:
+                view_kwargs.update(route.view_kwargs)
+            view_kwargs.update(result.kwargs)
+
+            if isinstance(route, DelegateRoute):
+                try:
+                    sub_routes = route.routecollectionfactory(*view_args,
+                                                              **view_kwargs)
+
+                except ResponseException as e:
+                    def raiser(*args, **kwargs):
+                        raise e
+                    yield RouteTraversal(Route('/', ALL_METHODS, raiser),
+                                         (), {},
+                                         [(self, '')])
+                    continue
+                # Dynamic routes exhaust their arguments when creating the sub
+                # RouteCollection.
+                if route.dynamic:
+                    view_args = ()
+                    view_kwargs = {}
+                for sub in sub_routes.get_routes(result.path_remaining,
+                                                 method):
+                    traversed = [(self, '')]
+                    traversed.extend((c, result.path_matched + p)
+                                     for c, p in sub.collections_traversed)
+                    yield RouteTraversal(sub.route,
+                                         view_args + sub.args,
+                                         dict(view_kwargs, **sub.kwargs),
+                                         traversed)
+            else:
+                yield RouteTraversal(route,
+                                     view_args, view_kwargs,
+                                     [(self, '')])
+
+    def route(self, pattern, methods, view=None, *args, **kwargs):
+        """
+        Match a URL pattern to a view function. Can be used as a function
+        decorator, in which case the ``view`` parameter should not be passed.
+
+        :param pattern: A string that can be compiled into a path pattern
+        :param methods: A list of HTTP methods the view is bound to
+        :param view:    The view function. If not specified a function
+                        decorator will be returned.
+
+        Other parameters are as for the :class:`Route` constructor.
+        """
+        # Called as a decorator?
+        if view is None:
+            def route_decorator(func):
+                self.add_route(Route(pattern, methods, func, *args, **kwargs))
+                return func
+            return route_decorator
+
+        route = Route(pattern, methods, view, *args, **kwargs)
+        self.add_route(route)
+        return route
+
+    def route_wsgi(self, path, wsgiapp, *args, **kwargs):
+
+        from fresco import context
+
+        def fresco_wsgi_view():
+            fake_start_response = lambda status, headers, exc_info=None: None
+            return Response.from_wsgi(wsgiapp,
+                                      context.request.environ,
+                                      fake_start_response)
+
+        return self.route_all(path, ALL_METHODS, fresco_wsgi_view, *args,
+                              **kwargs)
+
+    def route_all(self, path, *args, **kwargs):
+        """\
+        Expose a view for all URLs starting with ``path``.
+
+        :param path: the path prefix at which the view will be routed
+        """
+        return self.route(MatchAllURLsPattern(path), *args, **kwargs)
+
+    def include(self, path, views):
+        """
+        Include a view collection at the given path.
+
+        The included view collection's url properties will be modified to
+        generate the prefixed URLs.
+
+        :param path:  Path at which to include the views
+        :param views: Any collection of views (must expose a ``__routes___``
+                      attribute)
+        """
+        routes = list(r.add_prefix(path) for r in views.__routes__)
+        for r in routes:
+            r.resolve_view(views)
+            self.add_route(r)
+
+    def delegate(self, path, app, dynamic=False, *args, **kwargs):
+        """\
+        Delegate all requests under ``path`` to ``app``
+
+        :param path: the path prefix at which the app will be available
+        :param app: a FrescoApp instance
+        """
+        route = DelegateRoute(path, app, dynamic, *args, **kwargs)
+        self.add_route(route)
+        return route
+
+
+class DelegateRoute(Route):
+    """\
+    A specialisation of Route that is used to delegate a path prefix to another
+    route collection.
+    """
+
+    def __init__(self, prefix, view, dynamic=False,
+                 *args, **kwargs):
+        pattern = ExtensiblePattern(prefix, match_entire_path=False)
+        self.dynamic = dynamic
+        if self.dynamic:
+            self.routecollectionfactory = self._dynamic_routecollectionfactory
+        else:
+            self.routecollectionfactory = self._static_routecollectionfactory
+            if not isinstance(view, RouteCollection):
+                view = RouteCollection(view.__routes__)
+        super(DelegateRoute, self).__init__(pattern, ALL_METHODS,
+                                            view, *args, **kwargs)
+
+    def _dynamic_routecollectionfactory(self, *args, **kwargs):
+        """\
+        Return the RouteCollection  responsible for paths under this route and
+        the remaining positional+keyword args
+        """
+
+        cached = context.get(self)
+        if cached:
+            return cached
+
+        routes = self.view(*args, **kwargs)
+
+        if isinstance(routes, RouteCollection):
+            return routes
+
+        for r in routes.__routes__:
+            r.resolve_view(routes)
+
+        return RouteCollection(routes.__routes__)
+
+    def _static_routecollectionfactory(self, *args, **kwargs):
+        return self.view
 
 
 class RouteKwarg(object):
@@ -625,6 +963,7 @@ class RequestArg(RouteKwarg):
     query = attrgetter('query')
     form = attrgetter('form')
     cookies = attrgetter('cookies')
+    session = attrgetter('session')
 
     #: Source for the request variable
     source = form
@@ -683,7 +1022,7 @@ class CookieArg(RequestArg):
     Example::
 
         @app.route('/', GET, message=CookieArg(key='msg', default=None))
-        def view(request, message):
+        def view(message):
             return Response([message])
     """
     source = RequestArg.cookies
@@ -706,7 +1045,7 @@ class QueryArg(RequestArg):
     Example::
 
         @app.route('/add', GET, a=QueryArg(int), b=QueryArg(int))
-        def view(request, a, b):
+        def view(a, b):
             return Response(['a + b = %d' % (a + b)])
     """
     source = RequestArg.query
@@ -719,8 +1058,39 @@ class FormArg(RequestArg):
     Example::
 
         @app.route('/mul', POST, a=FormArg(int), b=FormArg(int))
-        def view(request, a, b):
+        def view(a, b):
             return Response(['a * b = %d' % (a * b)])
 
     """
     source = RequestArg.form
+
+
+class SessionArg(RequestArg):
+    """\
+    Extract a view keyword argument from ``request.form``.
+
+    Example::
+
+        @app.route('/mul', POST, a=FormArg(int), b=FormArg(int))
+        def view(a, b):
+            return Response(['a * b = %d' % (a * b)])
+
+    """
+    source = RequestArg.session
+
+
+class From(object):
+    """\
+    Accessors for RequestArg classes::
+
+        from fresco.routing import Route, From
+
+        Route('/blah', GET, x=From.get(), y=From.cookie())
+
+    """
+    form = FormArg
+    query = QueryArg
+    get = QueryArg
+    post = FormArg
+    cookie = CookieArg
+    session = SessionArg
