@@ -5,8 +5,8 @@ Utilities for dealing with static file content
 import mimetypes
 import os
 from email.utils import formatdate, parsedate_tz, mktime_tz
-from time import strftime, gmtime
 
+from fresco import context
 from fresco.exceptions import NotFound, Forbidden
 from fresco.response import Response, STATUS_OK, STATUS_NOT_MODIFIED
 from fresco.util.wsgi import ClosingIterator
@@ -14,7 +14,7 @@ from fresco.util.wsgi import ClosingIterator
 __all__ = 'serve_static_file',
 
 
-def serve_static_file(request, path, default_charset="ISO-8859-1",
+def serve_static_file(path, default_charset="ISO-8859-1",
                       bufsize=8192):
     """
     Serve a static file located at ``path``.
@@ -28,6 +28,7 @@ def serve_static_file(request, path, default_charset="ISO-8859-1",
     except OSError:
         raise NotFound()
 
+    request = context.request
     mod_since = request.get_header('if-modified-since')
     if mod_since is not None:
         mod_since = mktime_tz(parsedate_tz(mod_since))
