@@ -1,6 +1,9 @@
+# coding=UTF-8
+
 from nose.tools import assert_equal
 
 from fresco import FrescoApp
+from fresco.compat import PY3, quote
 
 context = FrescoApp().requestcontext
 
@@ -49,6 +52,46 @@ class TestRequestProperties(object):
             assert_equal(c.request.path, '/foo/bar')
 
 
+class TestPathEncoding(object):
+
+    if PY3:
+        unquoted_url = '/el niño'
+        quoted_url = quote(unquoted_url)
+        wsgi_url = unquoted_url.encode('utf8').decode('latin1')
+    else:
+        unquoted_url = unicode('/el niño', 'UTF-8')
+        quoted_url = quote(unquoted_url.encode('UTF-8'))
+        wsgi_url = unquoted_url.encode('UTF-8')
+
+    def test_url_is_quoted(self):
+
+        with context(SCRIPT_NAME=self.wsgi_url, PATH_INFO='') as c:
+            assert c.request.url == 'http://localhost' + self.quoted_url
+
+    def test_application_url_is_quoted(self):
+
+        with context(SCRIPT_NAME=self.wsgi_url, PATH_INFO='') as c:
+            assert c.request.application_url == 'http://localhost' + \
+                                                    self.quoted_url
+
+    def test_parsed_url_is_quoted(self):
+
+        with context(SCRIPT_NAME=self.wsgi_url, PATH_INFO='') as c:
+            assert c.request.parsed_url.path == self.quoted_url
+
+    def test_path_is_unquoted(self):
+        with context(SCRIPT_NAME=self.wsgi_url, PATH_INFO='') as c:
+            assert c.request.path == self.unquoted_url
+
+    def test_script_name_is_unquoted(self):
+        with context(SCRIPT_NAME=self.wsgi_url, PATH_INFO='') as c:
+            assert c.request.script_name == self.unquoted_url
+
+    def test_path_info_is_unquoted(self):
+        with context(SCRIPT_NAME='', PATH_INFO=self.wsgi_url) as c:
+            assert c.request.path_info == self.unquoted_url
+
+
 class TestMakeURL(object):
 
     def test_returns_request_url(self):
@@ -77,7 +120,6 @@ class TestMakeURL(object):
                          'http://localhost/?a=1;b=2')
 
 
-
 class TestResolveURL(object):
 
     def test_relative_path(self):
@@ -120,7 +162,7 @@ class TestCurrentRequest(object):
 
 #def test_parsing_cookie_header():
 #    @to_wsgi
-#    def app(request):
+#    def app():
 #        return Response([
 #            str(cookie) + '\n' for name, cookie in request.cookies.allitems()
 #        ])
@@ -136,7 +178,7 @@ class TestCurrentRequest(object):
 #    dispatcher = dispatcher_app()
 #
 #    @dispatcher.match('/blah', 'DELETE', 'PUT')
-#    def app(request):
+#    def app():
 #        return Response.method_not_allowed(request)
 #
 #    response = TestApp(dispatcher).get('/blah')
@@ -159,14 +201,14 @@ class TestCurrentRequest(object):
 #def test_form():
 #
 #    request = Request(make_environ(QUERY_STRING='foo=bar'))
-#    assert_equal(request.get('foo'), u'bar')
-#    assert_equal(request['foo'], u'bar')
+#    assert_equal(request.get('foo'), 'bar')
+#    assert_equal(request['foo'], 'bar')
 #    assert_equal(request.get('fuz'), None)
 #
 #    request = Request(make_environ(QUERY_STRING='foo=bar&foo=baz'))
-#    assert_equal(request.get('foo'), u'bar')
-#    assert_equal(request['foo'], u'bar')
-#    assert_equal(request.getlist('foo'), [u'bar', u'baz'])
+#    assert_equal(request.get('foo'), 'bar')
+#    assert_equal(request['foo'], 'bar')
+#    assert_equal(request.getlist('foo'), ['bar', u'baz'])
 #
 #def test_remote_addr():
 #    request = Request(make_environ(REMOTE_ADDR="1.2.3.4", HTTP_X_FORWARDED_FOR="4.3.2.1"))
@@ -176,24 +218,24 @@ class TestCurrentRequest(object):
 #def test_encoding():
 #
 #    # Test UTF-8 decoding works (this should be the default)
-#    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf8'))))
-#    assert_equal(request.get('foo'), u'\xa0')
+#    request = Request(make_environ(QUERY_STRING=make_query(foo='\xa0'.encode('utf8'))))
+#    assert_equal(request.get('foo'), '\xa0')
 #
 #    # Force a different encoding
-#    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf7'))))
+#    request = Request(make_environ(QUERY_STRING=make_query(foo='\xa0'.encode('utf7'))))
 #    request.charset = 'utf-7'
-#    assert_equal(request.get('foo'), u'\xa0')
+#    assert_equal(request.get('foo'), '\xa0')
 #
 #    # Force an incorrect encoding
-#    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf8'))))
+#    request = Request(make_environ(QUERY_STRING=make_query(foo='\xa0'.encode('utf8'))))
 #    request.charset = 'utf-7'
 #    assert_raises(RequestParseError, request.get, 'foo')
 #
 #def test_form_getters():
 #
-#    alpha = u'\u03b1' # Greek alpha
-#    beta = u'\u03b2' # Greek beta
-#    gamma = u'\u03b3' # Greek gamma
+#    alpha = '\u03b1' # Greek alpha
+#    beta = '\u03b2' # Greek beta
+#    gamma = '\u03b3' # Greek gamma
 #
 #    request = Request(make_environ(
 #        QUERY_STRING='foo=%s;foo=%s;bar=%s' % (
