@@ -1,10 +1,9 @@
 from fresco.util.urls import normpath, make_query
 
-# Some Greek letters as unicode strings (requires multi-byte representation
-# in UTF-8)
-alpha = u'\u03b1'
-beta = u'\u03b2'
-gamma = u'\u03b3'
+# Greek letters as unicode strings (require multi-byte representation in UTF-8)
+alpha = b'\xce\xb1'.decode('utf8')
+beta = b'\xce\xb2'.decode('utf8')
+gamma = b'\xce\xb3'.decode('utf8')
 
 
 class TestNormPath(object):
