@@ -7,11 +7,20 @@ See LICENSE.txt for terms of redistribution and use.
 
 import re
 from email.message import Message
-from email.parser import Parser
-from urllib import unquote_plus
+try:
+    from email.parser import BytesParser
+except ImportError:
+    # python 2
+    from email.parser import Parser as BytesParser  # NOQA
+
+try:
+    from urllib.parse import unquote_plus
+except ImportError:
+    from urllib import unquote_plus  # NOQA
 from shutil import copyfileobj
 
 import fresco
+from fresco.compat import PY3
 from fresco.exceptions import RequestParseError
 from fresco.util.io import ExpandableOutput, SizeLimitedInput, PutbackInput, \
                            DelimitedInput
@@ -84,8 +93,9 @@ def parse_querystring(data, charset=None, strict=False, keep_blank_values=True,
     Return ``(key, value)`` pairs from the given querystring::
 
         >>> list(parse_querystring('green%20eggs=ham;me=sam+i+am'))
-        [(u'green eggs', u'ham'), (u'me', u'sam i am')]
+        [('green eggs', u'ham'), (u'me', u'sam i am')]
 
+    :param data: The query string to parse.
     :param charset: Character encoding used to decode values. If not specified,
                     ``fresco.DEFAULT_CHARSET`` will be used.
 
@@ -100,6 +110,12 @@ def parse_querystring(data, charset=None, strict=False, keep_blank_values=True,
     if charset is None:
         charset = fresco.DEFAULT_CHARSET
 
+    if PY3:
+        unquote = lambda s: unquote_plus(s, charset)
+    else:
+        unquote = lambda s: unquote_plus(s).encode('iso-8859-1')\
+                                           .decode(charset)
+
     for item in pairsplitter(data):
         if not item:
             continue
@@ -113,8 +129,7 @@ def parse_querystring(data, charset=None, strict=False, keep_blank_values=True,
             key, value = item, ''
 
         try:
-            yield (unquote_plus(key).decode(charset),
-                   unquote_plus(value).decode(charset))
+            yield (unquote(key), unquote(value))
         except UnicodeDecodeError:
             raise RequestParseError("Invalid character data: can't decode"\
                                     " as %r" % (charset,))
@@ -154,8 +169,9 @@ def parse_post(environ, fp, default_charset=None, max_size=16 * KB,
     if ct == 'application/x-www-form-urlencoded':
         if content_length > max_size:
             raise TooBig("Content Length exceeds permitted size")
-        return parse_querystring(SizeLimitedInput(fp, content_length).read(),
-                                 charset)
+        return parse_querystring(SizeLimitedInput(fp, content_length)
+                                    .read()
+                                    .decode('ASCII'), charset)
     else:
         if content_length > max_multipart_size:
             raise TooBig("Content Length exceeds permitted size")
@@ -164,8 +180,8 @@ def parse_post(environ, fp, default_charset=None, max_size=16 * KB,
         except KeyError:
             raise RequestParseError("No boundary given in multipart/form-data"\
                                     " content-type")
-        return parse_multipart(SizeLimitedInput(fp, content_length), boundary,
-                               charset, max_size)
+        return parse_multipart(SizeLimitedInput(fp, content_length),
+                               boundary.encode('ascii'), charset, max_size)
 
 
 class HTTPMessage(Message):
@@ -194,7 +210,7 @@ def parse_multipart(fp, boundary, default_charset, max_size):
     """
 
     boundary_size = len(boundary)
-    if not boundary.startswith('--'):
+    if not boundary.startswith(b'--'):
         raise RequestParseError("Malformed boundary string:"\
                                 " must start with '--' (rfc 2046)")
 
@@ -206,13 +222,13 @@ def parse_multipart(fp, boundary, default_charset, max_size):
     assert boundary_size + 2 < CHUNK_SIZE, \
           "CHUNK_SIZE cannot be smaller than the boundary string"
 
-    if fp.read(2) != '--':
+    if fp.read(2) != b'--':
         raise RequestParseError("Malformed POST data: expected two hypens")
 
     if fp.read(boundary_size) != boundary:
         raise RequestParseError("Malformed POST data: expected boundary")
 
-    if fp.read(2) != '\r\n':
+    if fp.read(2) != b'\r\n':
         raise RequestParseError("Malformed POST data: expected CRLF")
 
     fp = PutbackInput(fp)
@@ -233,9 +249,9 @@ def parse_multipart(fp, boundary, default_charset, max_size):
         is_file_upload = 'Content-Type' in headers and 'filename' in params
 
         if is_file_upload:
-            stream = data._stream
-            stream.seek(0)
-            yield name, FileUpload(params['filename'], headers, stream)
+            storage = data.getstorage()
+            storage.seek(0)
+            yield name, FileUpload(params['filename'], headers, storage)
 
         else:
             charset = parse_header(headers.get('Content-Type', ''))[1]\
@@ -250,13 +266,13 @@ def parse_multipart(fp, boundary, default_charset, max_size):
                                         " as %r" % (charset,))
 
         chunk = fp.read(2)
-        if chunk == '--':
-            if fp.peek(3) != '\r\n':
+        if chunk == b'--':
+            if fp.peek(3) != b'\r\n':
                 raise RequestParseError("Expected terminating CRLF"\
                                         " at end of stream")
             break
 
-        if chunk != '\r\n':
+        if chunk != b'\r\n':
             raise RequestParseError("Expected CRLF after boundary")
 
 
@@ -275,13 +291,14 @@ def _read_multipart_field(fp, boundary):
     Note that this currently cannot handle nested multipart sections.
     """
     data = ExpandableOutput()
-    headers = Parser(_class=HTTPMessage).parse(
-        DelimitedInput(fp, '\r\n\r\n'),
+    headers = BytesParser(_class=HTTPMessage).parse(
+        DelimitedInput(fp, b'\r\n\r\n'),
         headersonly=True
     )
-    fp = DelimitedInput(fp, '\r\n--' + boundary)
+    fp = DelimitedInput(fp, b'\r\n--' + boundary)
+
     # XXX: handle base64 encoding etc
-    for chunk in iter(lambda: fp.read(CHUNK_SIZE), ''):
+    for chunk in iter(lambda: fp.read(CHUNK_SIZE), b''):
         data.write(chunk)
     data.flush()
 
